<?php

namespace Drupal\smart_faq_chatbot\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\commerce_product\Entity\ProductType;
use Drupal\Core\Database\Connection;
use Drupal\Core\Render\RendererInterface;

/**
 * Provides a Smart FAQ Chatbot block.
 *
 * @Block(
 *   id = "smart_chatbot_block",
 *   admin_label = @Translation("Smart FAQ Chatbot"),
 * )
 */
class SmartChatbotBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;
  protected EntityTypeManagerInterface $entityTypeManager;
  /**
   * Constructs a new SmartChatbotBlock instance.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $config = $this->configFactory->get('smart_faq_chatbot.settings');

    // Load all node_types and their names.
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $content_type_options = [];
    foreach ($content_types as $contentType) {
        $content_type_options[$contentType->id()] = $contentType->label();
    }

    // Load all taxonomy vocabularies and their terms.
    $taxonomy_options = [];
    $vocabularies = Vocabulary::loadMultiple();
    $term_storage = $this->entityTypeManager->getStorage('taxonomy_term');
    foreach ($vocabularies as $vocab_id => $vocab) {
      $terms = $term_storage->loadTree($vocab_id);
      foreach ($terms as $term) {
        $taxonomy_options[$vocab->label()][$term->tid] = $term->name;
      }
    }

    // Load commerce product types (if module is enabled).
    $product_type_options = [];
    if (\Drupal::moduleHandler()->moduleExists('commerce_product')) {
      $product_types = ProductType::loadMultiple();
      foreach ($product_types as $product_type_id => $product_type) {
        $product_type_options[$product_type_id] = $product_type->label();
      }
    }

    $connection = \Drupal::database();
    $results = $connection->select('smart_faq_chatbot_recent_questions', 'q')
      ->fields('q', ['question', 'answer', 'created'])
      ->orderBy('created', 'DESC')
      ->range(0, 5)
      ->execute()
      ->fetchAll();

    $faq_items = [];
    foreach ($results as $row) {
      $faq_items[] = [
        'question' => $row->question,
        'answer' => $row->answer,
        'created' => \Drupal::service('date.formatter')->format($row->created, 'short'),
      ];
    }

    $build = [
      '#theme' => 'chatbot_block',
      '#content_types' => $content_type_options,
      '#taxonomy_terms' => $taxonomy_options,
      '#product_types' => $product_type_options,
      '#faq_items' => $faq_items,
      '#attached' => [
        'library' => [
          'smart_faq_chatbot/chatbot',
        ],
        'drupalSettings' => [
          'smartFaqChatbot' => [
            'maxTokens' => $config->get('max_tokens'),
            'systemPrompt' => $config->get('system_prompt'),
          ],
        ],
      ],
    ];

    return $build;
  }

}
