<?php

namespace Drupal\smart_faq_chatbot\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\ClientInterface;

/**
 * Service to handle FAQ chat interaction with Gemini API.
 */
class SmartChatbotService {

  /**
   * HTTP Client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * Gemini API Key.
   *
   * @var string
   */
  protected string $apiKey;

  /**
   * Constructs the chatbot service.
   */
  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory) {
    $this->httpClient = $http_client;
    $config = $config_factory->get('smart_faq_chatbot.settings');
    $this->apiKey = $config->get('api_key');
  }

  /**
   * Get an AI-generated answer to the question with optional context.
   *
   * @param string $question
   *   The user's question.
   * @param string|null $content_type
   *   Optional node content type.
   * @param int|null $taxonomy_term
   *   Optional taxonomy term ID.
   * @param string|null $product_type
   *   Optional commerce product type.
   *
   * @return string
   *   The chatbot's response.
   */
  public function getAnswer(string $question, string $contentType = '', string $taxonomyTerm = '', string $productType = ''): string {
    $context_text = '';

    // Get Node Titles + Summaries + URLs
    if (!empty($contentType)) {
      $nodes = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties(['type' => $contentType]);
      $nodeSnippets = '';
      foreach (array_slice($nodes, 0, 10) as $node) {
        if ($node->access('view')) {
          $summary = '';
          if ($node->hasField('field_project_description') && !$node->get('field_project_description')->isEmpty()) {
            $summary = text_summary($node->get('field_project_description')->value, NULL, 200);
          }
          $url = $node->toUrl()->setAbsolute()->toString();
          $nodeSnippets .= "- **{$node->label()}**: {$summary} [Visit]({$url})\n";
        }
      }
      if ($nodeSnippets) {
        $contextParts[] = "**Project/Proposal Content:**\n" . $nodeSnippets;
      }
    }

    // Get Taxonomy Term Info
    if (!empty($taxonomyTerm)) {
      $term = \Drupal\taxonomy\Entity\Term::load($taxonomyTerm);
      if ($term && $term->access('view')) {
        $desc = $term->getDescription();
        $contextParts[] = "**Topic Context:**\n- {$term->label()}: " . strip_tags($desc);
      }
    }
    // Get Product Type Info
    if (!empty($productType)) {
      $products = \Drupal::entityTypeManager()->getStorage('commerce_product')->loadByProperties(['type' => $productType]);
      $productSnippets = '';
      foreach (array_slice($products, 0, 5) as $product) {
        $url = $product->toUrl()->setAbsolute()->toString();
        $productSnippets .= "- {$product->label()} [Visit]({$url})\n";
      }
      if ($productSnippets) {
        $contextParts[] = "**Available Products:**\n" . $productSnippets;
      }
    }

    // Build the prompt.
    $config = \Drupal::config('smart_faq_chatbot.settings');
    $siteName = $config->get('site_name');
    $siteDescription = $config->get('site_description');
    $siteFeatures = $config->get('site_features');
    $siteKeywords = $config->get('site_keywords');
    
    $siteContext = <<<EOT
    Site Overview:
    - Name: $siteName
    - Description: $siteDescription
    - Features: $siteFeatures
    - Tags: $siteKeywords
    EOT;

    $contextText = implode("\n\n", $contextParts);

    $full_prompt = <<<EOT
    You are a helpful AI FAQ assistant for a Drupal Site. {$siteContext}

    Only answer using the internal content provided below. If no relevant info exists, say no politely.

    Context:
    $contextText

    User Question:
    $question

    Provide a helpful, factual answer using the internal content.
    EOT;    
    
    try {
      $response = $this->httpClient->post("https://generativelanguage.googleapis.com/v1beta/models/gemini-2.0-flash:generateContent?key={$this->apiKey}", [
        'headers' => [
          'Content-Type' => 'application/json',
        ],
        'json' => [
          'contents' => [
            [
              'parts' => [
                ['text' => $full_prompt],
              ],
            ],
          ],
        ],
      ]);

      $data = json_decode($response->getBody()->getContents(), TRUE);
      
      $answer = $data['candidates'][0]['content']['parts'][0]['text'] ?? 'No response.';

      // Only store meaningful answers.
      $rejection_phrases = [
        'I am sorry, I cannot answer this question from the content provided.',
        'I’m sorry, but I couldn’t find relevant information.',
        'No relevant content found to answer this.',
      ];
      
      $should_store = TRUE;
      foreach ($rejection_phrases as $phrase) {
        if (stripos($answer, $phrase) !== FALSE) {
          $should_store = FALSE;
          break;
        }
      }
      
      if ($should_store) {
        \Drupal::database()->insert('smart_faq_chatbot_recent_questions')
          ->fields([
            'question' => $question,
            'answer' => $answer,
            'created' => \Drupal::time()->getCurrentTime(),
          ])
          ->execute();
      }

      return $data['candidates'][0]['content']['parts'][0]['text'] ?? 'Sorry, I could not generate a response.';
      
    }
    catch (\Exception $e) {
      \Drupal::logger('smart_faq_chatbot')->error('Gemini API error: @message', ['@message' => $e->getMessage()]);
      return 'There was an error generating the response. Please try again.';
    }
  }

}
